@@ -8,7 +8,7 @@
 #include "Components.h"
 
 Components::Components() {
-
+  pconn = &Components::computeConnected;
 }
 
 Components::~Components() {
@@ -17,13 +17,13 @@ Components::~Components() {
 
 void Components::calculate(Graph *graph)
 {
-	int n, h, i, j, k, l, g, isnew, loop, sizei, sizeg, d, x;
-	n = graph->nodelist.size();
+	int h, i, j, k, l, g, isnew, loop, sizei, sizeg, d, x;
+	n = graph->size(); // internal n
 
 	std::vector<std::vector<int> > complist;
 	complist.resize(n);
 
-	if(*graph->dbg) Rprintf("grouping, ");
+	if(graph->dbg) Rprintf("grouping, ");
 	for(i=0; i < n; i++)
 	{
 		complist.at(i).clear();
@@ -36,7 +36,7 @@ void Components::calculate(Graph *graph)
 
 	loop=1;
 	d=0;
-	if(*graph->dbg) Rprintf("sorting, ");
+	if(graph->dbg) Rprintf("sorting, ");
 	i=0;
 	while(loop)
 	{
@@ -81,7 +81,7 @@ void Components::calculate(Graph *graph)
 
 	componentlist.clear();
 
-	if(*graph->dbg) Rprintf("cleaning");
+	if(graph->dbg) Rprintf("cleaning");
 	x=0;
 	for(i=0; i< n ; i++)
 	{
@@ -92,7 +92,7 @@ void Components::calculate(Graph *graph)
 			x++;
 			for(j=0; j < (int)complist.at(i).size(); j++)
 			{
-				p->push_back(complist.at(i).at(j)+1);
+				p->push_back(complist.at(i).at(j));
 				complist.at(complist.at(i).at(j)).clear();
 
 			}
@@ -100,26 +100,46 @@ void Components::calculate(Graph *graph)
 		}
 //		delete p;
 	}
-	if(*graph->dbg) Rprintf(" (x=%i)ok ",x);
+	if(graph->dbg) Rprintf(" (x=%i)ok ",x);
+  
+  
 }
 
 
-int Components::connected(int *i, int *j)
+int Components::computeConnected(int *i, int *j)
 {
 	int k,l,m;
-	//for(k=0; k < (int)connections.at(*i).size(); k++)
-	//	if( connections.at(*i).at(k) == *j+1) return 1;
 	for(k = 0; k < (int) componentlist.size(); k++)
 	{
 		for(l=0; l < (int) componentlist.at(k).size();l++)
-			if( componentlist.at(k).at(l)== (*i+1) )
+			if( componentlist.at(k).at(l)== (*i) )
 				for (m = 0; m < (int)componentlist.at(k).size(); ++m)
 				{
-					if(componentlist.at(k).at(m) == (*j+1))	{return 1;}
+					if(componentlist.at(k).at(m) == (*j))	{return 1;}
 				}
 	}
 	return 0;
 }
+/********************************************************************************************/
+int Components::connectionsPrecalculated(int *i, int *j){
+  if(*i==*j) return 0.0;
+	if(*i>*j) return connectionsPrecalculated(j, i);
+	return connectionTriangle[ *j-*i -1 + (int)((*i)*n-(*i)*(*i+1)/2) ];  
+}
+
+/********************************************************************************************/
+int Components::connected(int *i, int *j){
+  return (this->*pconn)(i,j);
+}
+
+/********************************************************************************************/
+void Components::preComputeConnected() {
+  connectionTriangle.resize( n*(n-1)/2);
+  int i,j;
+  for(i=0; i < n-1;i++)
+    for(j=i+1; j < n; j++) connectionTriangle[ j-i -1 + (int)((i)*n-(i)*(i+1)/2) ] = connected(&i,&j);
+  pconn = &Components::connectionsPrecalculated;
+}
 
 
 /********************************************************************************************/
