@@ -12,13 +12,19 @@
 #define COMPONENTS_H_
 
 class Components {
+  int n;
+  std::vector <int> connectionTriangle;
+  int (Components::*pconn)(int*, int*);
+  int connectionsPrecalculated(int *, int *);
+  int computeConnected(int *, int *);
 public:
 	Components();
 	virtual ~Components();
 	std::vector<std::vector<int> > componentlist;
 	void calculate(Graph *graph);
 	int connected(int *, int *);
-	SEXP toSEXP();
+  void preComputeConnected();
+  SEXP toSEXP();
 
 };
 
